#!/bin/bash

a=$1
b=$2
e=$3
f=$4

if [[ -z $a || -z $b || -z $e || -z $f ]]
then
    echo "usage: invariant.bash <a> <b> <e> <f>"
    exit 1
fi

eq=/Users/dennydahl/p-adic/roots/eq.bash
gcd=/Users/dennydahl/abelian/Bezout/gcd

set $( $eq -f '2x - 1' -p 17 )

half=$3

set $(

echo "

r1 = 65087437953

half = $half

ua = 1
ub = r1
ue = (half * r1^2) % (17^10)
uf = (half * r1^3) % (17^10)

a = $a
b = $b
e = $e
f = $f

for (i=0; i<4; ++i) {
    g[i] = (a*ua + b*ub + e*ue + f*uf + 17^20) % (17^10)

    at = a + 4*e
    bt = 3*b + 10*f
    et = -e
    ft = -b - 3*f

    a = at
    b = bt
    e = et
    f = ft

    "'

    print " g[", i, "]: ", g[i], "\n"
}

num_f1 = ( g[0] - g[2] + 17^10 ) % (17^10)
num_f2 = ( g[1] - g[3] + 17^10 ) % (17^10)
num    = ( num_f1 * num_f1 * num_f2 * num_f2 ) % (17^10)

print "num_f1: ", num_f1, "\n"
print "num_f2: ", num_f2, "\n"

den_f1 = ( g[0] - g[1] + 17^10 ) % (17^10)
den_f2 = ( g[1] - g[2] + 17^10 ) % (17^10)
den_f3 = ( g[2] - g[3] + 17^10 ) % (17^10)
den_f4 = ( g[3] - g[0] + 17^10 ) % (17^10)
den    = ( den_f1 * den_f2 * den_f3 * den_f4 ) % (17^10)

print "den_f1: ", den_f1, "\n"
print "den_f2: ", den_f2, "\n"
print "den_f3: ", den_f3, "\n"
print "den_f4: ", den_f4, "\n"

define display(x) {

    a9 = x % (prime^9)
    a10 = x % (prime^10)

    if (a9 == a10) {
	print x % (prime^10), "\n"
    }

    b9 = (prime^9) - (x % prime^9)
    b10 = (prime^10) - (x % prime^10)

    if (b9 == b10) {
	print "-", (prime^10) - (x % prime^10), "\n"
    }
}

prime = 17

print "num: "
ignore = display(num)

print "den: "
ignore = display(den)

' |

bc -q

)

for ((i=1; i<=24; ++i))
do
    eval token=\${$i}
    arg[i]=$token
done

for ((i=1; i<=24; ++i))
do
    case ${arg[i]} in
	num:) num=${arg[i+1]};;
	den:) den=${arg[i+1]};;
	g*) echo "${arg[i]%:} = ${arg[i+1]}";;
    esac
done

echo "numerator = $num"
echo "denominator = $den"

if [[ $num -ne 0 || $den -ne 0 ]]
then
    [[ $num -ge 0 ]] && abs_num=$num || abs_num=$(( -num ))
    [[ $den -ge 0 ]] && abs_den=$den || abs_den=$(( -den ))
    g=$( $gcd $abs_num $abs_den )
else
    g=1
fi

echo "gcd = $g"

num=$(( num / g ))
den=$(( den / g ))

if [[ $den < 0 ]]
then
    den=$(( -den ))
    num=$(( -num ))
fi

echo "invariant = $num/$den"
